#include "Application.h"
#include "texturesdl.h"

using namespace EPOC;

Application::Application()
{
	SetClassID(kCLASSID_APPLICATION);

	m_pContentManager = NULL;
	m_pGraphicsManager = NULL;
	m_pAudioManager = NULL;
	m_pInputManager = NULL;
}

Application::~Application()
{
	Application::Cleanup();
}


void Application::Cleanup()
{
	if(m_pContentManager)
		m_pContentManager->Release();
	m_pContentManager = NULL;

	if(m_pInputManager)
		m_pInputManager->Release();
	m_pInputManager = NULL;

	if(m_pAudioManager)
		m_pAudioManager->Release();
	m_pAudioManager = NULL;
	
	if(m_pGraphicsManager)
		m_pGraphicsManager->Release();
	m_pGraphicsManager = NULL;
}


bool Application::Initialize()
{
	if( CreateGraphicsManager() == false)
		return false;

	if(m_bAudioSupport )
	{
		if( CreateAudioManager() == false)
		{
			fprintf( stderr, "Application: Error, audio disabled\n");
			m_bAudioSupport = false;//return false;
		}
	}

	if( CreateInputManager() == false)
		return false;

	if( CreateContentManager() == false)
		return false;

	return true;
}


bool Application::CreateGraphicsManager()
{
	m_pGraphicsManager = new GraphicsManager(this);
	if(!m_pGraphicsManager || !m_pGraphicsManager->Initialize())
	{
		m_pGraphicsManager->Release();
		m_pGraphicsManager = NULL;

		return false;
	}

	return true;
}

bool Application::CreateAudioManager()
{
	m_pAudioManager = new AudioManager(this);
	if(!m_pAudioManager || !m_pAudioManager->Initialize(32, 44100, 2048))
	{
		m_pAudioManager->Release();
		m_pAudioManager = NULL;

		return false;
	}

	return true;
}

bool Application::CreateInputManager()
{
	m_pInputManager = new InputManager(this);
	if(!m_pInputManager || !m_pInputManager->Initialize())
	{
		m_pInputManager->Release();
		m_pInputManager = NULL;

		return false;
	}

	return true;
}

bool Application::CreateContentManager()
{
	m_pContentManager = new ContentManager(this);
	if(!m_pContentManager || !m_pContentManager->Initialize())
	{
		m_pContentManager->Release();
		m_pContentManager = NULL;

		return false;
	}

	return true;
}


bool Application::Update(float fTimeDelta)
{
	if(m_pContentManager)
	{
		if(m_pContentManager->Update(fTimeDelta) == false)
		{
			m_pContentManager->Release();
			m_pContentManager = NULL;

			return false;
		}
	}

	if(m_pGraphicsManager)
	{
		if(m_pGraphicsManager->Update(fTimeDelta) == false)
		{
			m_pGraphicsManager->Release();
			m_pGraphicsManager = NULL;

			return false;
		}
	}

	if(m_pAudioManager)
	{
		if(m_pAudioManager->Update(fTimeDelta) == false)
		{
			m_pAudioManager->Release();
			m_pAudioManager = NULL;

			return false;
		}
	}

	if(m_pInputManager)
	{
		if(m_pInputManager->Update(fTimeDelta) == false)
		{
			m_pInputManager->Release();
			m_pInputManager = NULL;

			return false;
		}
	}

	return true;
}

void Application::Render(TextureSDL* pDestSurface)
{
	if(m_pGraphicsManager)
		m_pGraphicsManager->Render(pDestSurface);
}


void Application::HandleEvent(SDL_Event *pEvent)
{
	if(m_pInputManager)
		m_pInputManager->HandleEvent(pEvent);
}